/**
 * 
 */
package objects;



import java.util.Stack;
import objects.nononjects.*;

/**
 * This class manages the "Transaction Values" (Magnetic force between the atoms)
 * 
 * This class is a "singleton" which make it impossible to define more than one
 * instance of this class. There will be only one object every time you call the the 
 * class with WWManager.getInstance(). 
 * 
 * This makes the class protected against threads which try to set Values at one (race condition)
 * or have different values caused by more than one object.
 *
 * @see objects.nonobjects.WechselWirkung
 */
public final class WWManager{
	
	
	
    /** Privates Klassenattribut, einzige Instanz der Klasse wird erzeugt. */
    private static final WWManager INSTANCE = new WWManager();

    private Stack<WechselWirkung> jj2ww = new Stack<WechselWirkung>();
    
    /** Konstruktor ist privat, darf nicht von außen instanziiert werden. */ 
    private WWManager() {}

    /** Statische Methode "getInstance()" liefert die einzige Instanz der Klasse zurück. */
    public static WWManager getInstance() {
        return INSTANCE;
    }
    /**
     * Checks for existans of a Transaction Index
     * @param jj	Transaction index
     * @return	if allready set -> true, else false
     */
    public boolean contains(int jj){
    	for (WechselWirkung ww : jj2ww) {
			if(ww.getIdentifier() == jj)
				return true;
		}
    	return false;
    }
    
    /**
     * get the transaction by indexnumber 
     * @param 	jj	transaction index number
     * @return 	transaction object
     */
    private WechselWirkung getPack(int jj){
    	for (WechselWirkung ww : jj2ww) {
			if(ww.getIdentifier() == jj) return ww;
		}
    	return null;
    }
    
    /**
     * Sets a new transaction to a specified Atom
     * @param jj	transaction index (if non exists a new one will be created)
     * @param a		atom which gets this new transaction value
     */
    public void addJJ(int jj, Atom a){
    	if(!contains(jj)){
    		WechselWirkung www = new WechselWirkung(jj,0);
    		www.addConnection(a);
    		jj2ww.add(www);
    	} else {
    		getPack(jj).addConnection(a);
    	}
    	
    	if(a != null){
    		for (WechselWirkung w : jj2ww) {
				if(w.getIdentifier() == jj){
					w.addConnection(a);
				}
			}
    	}
    	
    	
    }
    
    /**
     * Addes a new Transaction Value
     * @param jj	transaction index value
     */
    public void addJJ (int jj){
    	addJJ(jj, null);
    }
    
    /**
     * Define a new Transaction with a specified value for the transaction
     * @param jj	new transaction index
     * @param ww	transaction value
     */
    public void setWW(int jj, Float ww){
    	addJJ(jj);
    	getPack(jj).setValue(ww);   		
    }
    
    /**
     * Converts transaction index to transactionvalue
     * @param jj	transaction index
     * @return		transaction value
     */
    public Float getWW(int jj){
    	addJJ(jj);
    	return getPack(jj).getValue();
    }
    
    /**
     * @deprecated
     * @param jj
     * @param a
     * @return
     */
    /*public WechselWirkung getWW(int jj, Atom a){
    	getPack(jj).addConnection(a);
    	return getPack(jj);
    }*/
    
    /**
     * returns all collected transactions
     * @see objects.nonobjects.WechselWirkung 
     */
    public Stack<WechselWirkung> getWW(){
    	return jj2ww;
    }
    
    /**
     * clears all collected transactions
     *
     */
    public void clear(){
    	jj2ww.clear();
    }
    

}

